// tempus 2000.11
#ifndef TUTF_SYSTEM_CLASS
#define TUTF_SYSTEM_CLASS
//
// (c) Copyright 1995-2000, MZA Associates Corporation
//

#include "tempus.h"


#include "Telescope.h"
#include "TargetBoard.h"
#include "UniformWave.h"
#include "WindAtmoPath.h"
#include "PointSource.h"
#include "Camera.h"
#include "BeamSteeringMirror.h"
#include "Centroid.h"
#include "TriggeredWeightedSum.h"
#include "IncomingSplitter.h"
#include "Camera.h"
#include "IncomingCombiner.h"
#include "IncoherentReflector.h"
#include "UniformWave.h"
#include "Centroid.h"
#include "Centroid.h"

class TutF : public System
{
public:

   // Parameters
   float tel_focus_range;
   float tel_diam;
   float hel_wavelength;
   float ptsc_wavelength;
   float targ_wavelength;
   float ptsc_x;
   float ptsc_y;
   AcsAtmSpec AtmSpec;
   int AtmSeed;
   float x_wind;
   float y_wind;
   int propnxy;
   float propdxy;
   int tbd_nxy;
   float tbd_dxy;
   int img_nxy;
   float img_dxy;
   float bsm_timeConstant;
   float trk_gain;
   int num_speck;
   int SpeckSeed;
   Grid<float>& reflectance;

   // Subsystems
   Telescope telescope;

   TargetBoard targetboard;

   UniformWave uw;

   WindAtmoPath atmosphericpath;

   PointSource pointsource;

   Camera trk_camera;

   BeamSteeringMirror BSM;

   Centroid trk_centroid;

   TriggeredWeightedSum trk_command;

   IncomingSplitter splitter;

   Camera img_camera;

   IncomingCombiner source_combiner;

   IncoherentReflector incoherentreflector;

   UniformWave uw2;

   Centroid img_centroid;

   Centroid tbd_centroid;


   // Inputs

   // Outputs

private:      // Setting expressions for unsatisfied inputs/external outputs
   Output< bool >  targetboard_on;
   Output< double >  targetboard_exposureInterval;
   Output< double >  targetboard_exposureLength;
   Output< double >  targetboard_sampleInterval;
   Output< bool >  trk_camera_on;
   Output< double >  trk_camera_exposureInterval;
   Output< double >  trk_camera_exposureLength;
   Output< double >  trk_camera_sampleInterval;
   Output< WaveTrain >  splitter_outgoingIncident;
   Output< bool >  img_camera_on;
   Output< double >  img_camera_exposureInterval;
   Output< double >  img_camera_exposureLength;
   Output< double >  img_camera_sampleInterval;
   Output< WaveTrain >  source_combiner_outgoingIncident;

public:
   TutF(SystemNode *parent, char *name,
        float _tel_focus_range,
        float _tel_diam,
        float _hel_wavelength,
        float _ptsc_wavelength,
        float _targ_wavelength,
        float _ptsc_x,
        float _ptsc_y,
        AcsAtmSpec _AtmSpec,
        int _AtmSeed,
        float _x_wind,
        float _y_wind,
        int _propnxy,
        float _propdxy,
        int _tbd_nxy,
        float _tbd_dxy,
        int _img_nxy,
        float _img_dxy,
        float _bsm_timeConstant,
        float _trk_gain,
        int _num_speck,
        int _SpeckSeed,
        Grid<float>& _reflectance) : 
      System(parent, name),
      tel_focus_range(_tel_focus_range),
      tel_diam(_tel_diam),
      hel_wavelength(_hel_wavelength),
      ptsc_wavelength(_ptsc_wavelength),
      targ_wavelength(_targ_wavelength),
      ptsc_x(_ptsc_x),
      ptsc_y(_ptsc_y),
      AtmSpec(_AtmSpec),
      AtmSeed(_AtmSeed),
      x_wind(_x_wind),
      y_wind(_y_wind),
      propnxy(_propnxy),
      propdxy(_propdxy),
      tbd_nxy(_tbd_nxy),
      tbd_dxy(_tbd_dxy),
      img_nxy(_img_nxy),
      img_dxy(_img_dxy),
      bsm_timeConstant(_bsm_timeConstant),
      trk_gain(_trk_gain),
      num_speck(_num_speck),
      SpeckSeed(_SpeckSeed),
      reflectance(_reflectance),
      telescope(this, "telescope",
         tel_focus_range,
         tel_diam/2.0,
         0.0),
      targetboard(this, "targetboard",
         hel_wavelength,
         tbd_nxy,
         tbd_dxy),
      uw(this, "uw",
         hel_wavelength,
         hel_wavelength,
         1.0),
      atmosphericpath(this, "atmosphericpath",
         AtmSpec,
         AtmSeed,
         propnxy,
         propdxy,
         x_wind,
         y_wind),
      pointsource(this, "pointsource",
         ptsc_wavelength,
         1.0e6,
         ptsc_x,
         ptsc_y),
      trk_camera(this, "trk_camera",
         1.0,
         targ_wavelength,
         targ_wavelength,
         tel_diam / propdxy,
         propdxy,
         img_nxy,
         img_dxy,
         0.0),
      BSM(this, "BSM",
         0.0,
         bsm_timeConstant),
      trk_centroid(this, "trk_centroid"),
      trk_command(this, "trk_command",
         -trk_gain,
         1.0),
      splitter(this, "splitter"),
      img_camera(this, "img_camera",
         1.0,
         ptsc_wavelength,
         ptsc_wavelength,
         tel_diam / propdxy,
         propdxy,
         img_nxy,
         img_dxy,
         0.0),
      source_combiner(this, "source_combiner"),
      incoherentreflector(this, "incoherentreflector",
         reflectance,
         propdxy,
         num_speck,
         SpeckSeed),
      uw2(this, "uw2",
         targ_wavelength,
         targ_wavelength,
         1.0),
      img_centroid(this, "img_centroid"),
      tbd_centroid(this, "tbd_centroid"),
      targetboard_on(this, "targetboard_on"),
      targetboard_exposureInterval(this, "targetboard_exposureInterval"),
      targetboard_exposureLength(this, "targetboard_exposureLength"),
      targetboard_sampleInterval(this, "targetboard_sampleInterval"),
      trk_camera_on(this, "trk_camera_on"),
      trk_camera_exposureInterval(this, "trk_camera_exposureInterval"),
      trk_camera_exposureLength(this, "trk_camera_exposureLength"),
      trk_camera_sampleInterval(this, "trk_camera_sampleInterval"),
      splitter_outgoingIncident(this, "splitter_outgoingIncident"),
      img_camera_on(this, "img_camera_on"),
      img_camera_exposureInterval(this, "img_camera_exposureInterval"),
      img_camera_exposureLength(this, "img_camera_exposureLength"),
      img_camera_sampleInterval(this, "img_camera_sampleInterval"),
      source_combiner_outgoingIncident(this, "source_combiner_outgoingIncident")
   {
      atmosphericpath.outgoingincident <<= telescope.outgoingTransmitted;
      targetboard.incident <<= atmosphericpath.outgoingtransmitted;
      telescope.incomingIncident <<= atmosphericpath.incomingtransmitted;
      BSM.outgoingIncident <<= uw.transmitted;
      telescope.outgoingIncident <<= BSM.outgoingTransmitted;
      BSM.incomingIncident <<= telescope.incomingTransmitted;
      trk_centroid.grid <<= trk_camera.fpaImage;
      BSM.tiltCommands <<= trk_command.sum;
      splitter.incomingIncident <<= BSM.incomingTransmitted;
      trk_camera.incident <<= splitter.incomingTransmitted;
      img_camera.incident <<= splitter.incomingTransmitted2;
      source_combiner.incomingIncident <<= pointsource.transmitted;
      atmosphericpath.incomingincident <<= source_combiner.incomingTransmitted;
      source_combiner.incomingIncident2 <<= incoherentreflector.transmitted;
      incoherentreflector.incident <<= uw2.transmitted;
      trk_command.summand2 <<= BSM.measuredTilts;
      trk_command.summand1 <<= trk_centroid.v;
      img_centroid.grid <<= img_camera.fpaImage;
      tbd_centroid.grid <<= targetboard.integrated_intensity;

      // System's Outputs and Subsystems' Inputs that
      // are not connected, but have setting expressions
      targetboard.on <<= targetboard_on;
      targetboard.exposureInterval <<= targetboard_exposureInterval;
      targetboard.exposureLength <<= targetboard_exposureLength;
      targetboard.sampleInterval <<= targetboard_sampleInterval;
      trk_camera.on <<= trk_camera_on;
      trk_camera.exposureInterval <<= trk_camera_exposureInterval;
      trk_camera.exposureLength <<= trk_camera_exposureLength;
      trk_camera.sampleInterval <<= trk_camera_sampleInterval;
      splitter.outgoingIncident <<= splitter_outgoingIncident;
      img_camera.on <<= img_camera_on;
      img_camera.exposureInterval <<= img_camera_exposureInterval;
      img_camera.exposureLength <<= img_camera_exposureLength;
      img_camera.sampleInterval <<= img_camera_sampleInterval;
      source_combiner.outgoingIncident <<= source_combiner_outgoingIncident;
   }

   // void respondToScheduledEvent (const Event & /* event */);
   // void respondToChangedInputs();
   // void respondToInputWarning (const InputBase & /* input */);
   void respondToOutputRequest (const OutputBase * output)
   {
      double t = now();
      if(*output == targetboard_on)
      {
         targetboard_on = (bool)(true);
      }
      else if(*output == targetboard_exposureInterval)
      {
         targetboard_exposureInterval = (double)(1.0e-3);
      }
      else if(*output == targetboard_exposureLength)
      {
         targetboard_exposureLength = (double)(1.0e-6);
      }
      else if(*output == targetboard_sampleInterval)
      {
         targetboard_sampleInterval = (double)(0.0);
      }
      else if(*output == trk_camera_on)
      {
         trk_camera_on = (bool)(true);
      }
      else if(*output == trk_camera_exposureInterval)
      {
         trk_camera_exposureInterval = (double)(1.0e-3);
      }
      else if(*output == trk_camera_exposureLength)
      {
         trk_camera_exposureLength = (double)(1.0e-6);
      }
      else if(*output == trk_camera_sampleInterval)
      {
         trk_camera_sampleInterval = (double)(0.0);
      }
      else if(*output == splitter_outgoingIncident)
      {
         splitter_outgoingIncident = (WaveTrain)(WaveTrain());
      }
      else if(*output == img_camera_on)
      {
         img_camera_on = (bool)(true);
      }
      else if(*output == img_camera_exposureInterval)
      {
         img_camera_exposureInterval = (double)(1.0e-3);
      }
      else if(*output == img_camera_exposureLength)
      {
         img_camera_exposureLength = (double)(1.0e-6);
      }
      else if(*output == img_camera_sampleInterval)
      {
         img_camera_sampleInterval = (double)(0.0);
      }
      else if(*output == source_combiner_outgoingIncident)
      {
         source_combiner_outgoingIncident = (WaveTrain)(WaveTrain());
      }
   }
};

#endif // TUTF_SYSTEM_CLASS
