// tempus 2000.11
#ifndef TUTI_SYSTEM_CLASS
#define TUTI_SYSTEM_CLASS
//
// (c) Copyright 1995-2000, MZA Associates Corporation
//

#include "tempus.h"


#include "Telescope.h"
#include "TargetBoard.h"
#include "UniformWave.h"
#include "WindAtmoPath.h"
#include "PointSource.h"
#include "Camera.h"
#include "BeamSteeringMirror.h"
#include "Centroid.h"
#include "TriggeredWeightedSum.h"
#include "Centroid.h"
#include "IncomingSplitter.h"
#include "WFSandRecon.h"
#include "DeformableMirror.h"
#include "TriggeredWeightedSum.h"

class TutI : public System
{
public:

   // Parameters
   float tel_focus_range;
   float tel_diam;
   float hel_wavelength;
   float ptsc_wavelength;
   float ptsc_x;
   float ptsc_y;
   AcsAtmSpec AtmSpec;
   int AtmSeed;
   float x_wind;
   float y_wind;
   int propnxy;
   float propdxy;
   int tbd_nxy;
   float tbd_dxy;
   int img_nxy;
   float img_dxy;
   float bsm_timeConstant;
   float trk_gain;
   float AO_gain;
   float wfs_ifov;
   DMModel& dmModel;

   // Subsystems
   Telescope telescope;

   TargetBoard targetboard;

   UniformWave uw;

   WindAtmoPath atmosphericpath;

   PointSource pointsource;

   Camera camera;

   BeamSteeringMirror BSM;

   Centroid trk_centroid;

   TriggeredWeightedSum trk_command;

   Centroid tbd_centroid;

   IncomingSplitter splitter;

   WFSandRecon wfsandrecon;

   DeformableMirror deformablemirror;

   TriggeredWeightedSum AO_command;


   // Inputs

   // Outputs

private:      // Setting expressions for unsatisfied inputs/external outputs
   Output< bool >  targetboard_on;
   Output< double >  targetboard_exposureInterval;
   Output< double >  targetboard_exposureLength;
   Output< double >  targetboard_sampleInterval;
   Output< bool >  camera_on;
   Output< double >  camera_exposureInterval;
   Output< double >  camera_exposureLength;
   Output< double >  camera_sampleInterval;
   Output< WaveTrain >  splitter_outgoingIncident;

public:
   TutI(SystemNode *parent, char *name,
        float _tel_focus_range,
        float _tel_diam,
        float _hel_wavelength,
        float _ptsc_wavelength,
        float _ptsc_x,
        float _ptsc_y,
        AcsAtmSpec _AtmSpec,
        int _AtmSeed,
        float _x_wind,
        float _y_wind,
        int _propnxy,
        float _propdxy,
        int _tbd_nxy,
        float _tbd_dxy,
        int _img_nxy,
        float _img_dxy,
        float _bsm_timeConstant,
        float _trk_gain,
        float _AO_gain,
        float _wfs_ifov,
        DMModel& _dmModel) : 
      System(parent, name),
      tel_focus_range(_tel_focus_range),
      tel_diam(_tel_diam),
      hel_wavelength(_hel_wavelength),
      ptsc_wavelength(_ptsc_wavelength),
      ptsc_x(_ptsc_x),
      ptsc_y(_ptsc_y),
      AtmSpec(_AtmSpec),
      AtmSeed(_AtmSeed),
      x_wind(_x_wind),
      y_wind(_y_wind),
      propnxy(_propnxy),
      propdxy(_propdxy),
      tbd_nxy(_tbd_nxy),
      tbd_dxy(_tbd_dxy),
      img_nxy(_img_nxy),
      img_dxy(_img_dxy),
      bsm_timeConstant(_bsm_timeConstant),
      trk_gain(_trk_gain),
      AO_gain(_AO_gain),
      wfs_ifov(_wfs_ifov),
      dmModel(_dmModel),
      telescope(this, "telescope",
         tel_focus_range,
         tel_diam/2.0,
         0.0),
      targetboard(this, "targetboard",
         hel_wavelength,
         tbd_nxy,
         tbd_dxy),
      uw(this, "uw",
         hel_wavelength,
         hel_wavelength,
         1.0),
      atmosphericpath(this, "atmosphericpath",
         AtmSpec,
         AtmSeed,
         propnxy,
         propdxy,
         x_wind,
         y_wind),
      pointsource(this, "pointsource",
         ptsc_wavelength,
         1.0e6,
         ptsc_x,
         ptsc_y),
      camera(this, "camera",
         1.0,
         ptsc_wavelength,
         ptsc_wavelength,
         tel_diam / propdxy,
         propdxy,
         img_nxy,
         img_dxy,
         0.0),
      BSM(this, "BSM",
         0.0,
         bsm_timeConstant),
      trk_centroid(this, "trk_centroid"),
      trk_command(this, "trk_command",
         -trk_gain,
         1.0),
      tbd_centroid(this, "tbd_centroid"),
      splitter(this, "splitter"),
      wfsandrecon(this, "wfsandrecon",
         dmModel,
         propnxy,
         propdxy,
         ptsc_wavelength,
         2,
         wfs_ifov),
      deformablemirror(this, "deformablemirror",
         dmModel,
         0.0,
         1.e-7),
      AO_command(this, "AO_command",
         AO_gain,
         1.0),
      targetboard_on(this, "targetboard_on"),
      targetboard_exposureInterval(this, "targetboard_exposureInterval"),
      targetboard_exposureLength(this, "targetboard_exposureLength"),
      targetboard_sampleInterval(this, "targetboard_sampleInterval"),
      camera_on(this, "camera_on"),
      camera_exposureInterval(this, "camera_exposureInterval"),
      camera_exposureLength(this, "camera_exposureLength"),
      camera_sampleInterval(this, "camera_sampleInterval"),
      splitter_outgoingIncident(this, "splitter_outgoingIncident")
   {
      atmosphericpath.outgoingincident <<= telescope.outgoingTransmitted;
      targetboard.incident <<= atmosphericpath.outgoingtransmitted;
      telescope.incomingIncident <<= atmosphericpath.incomingtransmitted;
      atmosphericpath.incomingincident <<= pointsource.transmitted;
      telescope.outgoingIncident <<= BSM.outgoingTransmitted;
      BSM.incomingIncident <<= telescope.incomingTransmitted;
      trk_centroid.grid <<= camera.fpaImage;
      trk_command.summand1 <<= trk_centroid.v;
      trk_command.summand2 <<= BSM.measuredTilts;
      BSM.tiltCommands <<= trk_command.sum;
      tbd_centroid.grid <<= targetboard.integrated_intensity;
      camera.incident <<= splitter.incomingTransmitted;
      wfsandrecon.incident <<= splitter.incomingTransmitted2;
      deformablemirror.incomingIncident <<= BSM.incomingTransmitted;
      BSM.outgoingIncident <<= deformablemirror.outgoingTransmitted;
      deformablemirror.outgoingIncident <<= uw.transmitted;
      splitter.incomingIncident <<= deformablemirror.incomingTransmitted;
      AO_command.summand1 <<= wfsandrecon.commands;
      AO_command.summand2 <<= deformablemirror.measuredActuatorPositions;
      deformablemirror.actuatorCommands <<= AO_command.sum;

      // System's Outputs and Subsystems' Inputs that
      // are not connected, but have setting expressions
      targetboard.on <<= targetboard_on;
      targetboard.exposureInterval <<= targetboard_exposureInterval;
      targetboard.exposureLength <<= targetboard_exposureLength;
      targetboard.sampleInterval <<= targetboard_sampleInterval;
      camera.on <<= camera_on;
      camera.exposureInterval <<= camera_exposureInterval;
      camera.exposureLength <<= camera_exposureLength;
      camera.sampleInterval <<= camera_sampleInterval;
      splitter.outgoingIncident <<= splitter_outgoingIncident;
   }

   // void respondToScheduledEvent (const Event & /* event */);
   // void respondToChangedInputs();
   // void respondToInputWarning (const InputBase & /* input */);
   void respondToOutputRequest (const OutputBase * output)
   {
      double t = now();
      if(*output == targetboard_on)
      {
         targetboard_on = (bool)(true);
      }
      else if(*output == targetboard_exposureInterval)
      {
         targetboard_exposureInterval = (double)(1.0e-3);
      }
      else if(*output == targetboard_exposureLength)
      {
         targetboard_exposureLength = (double)(1.0e-6);
      }
      else if(*output == targetboard_sampleInterval)
      {
         targetboard_sampleInterval = (double)(0.0);
      }
      else if(*output == camera_on)
      {
         camera_on = (bool)(true);
      }
      else if(*output == camera_exposureInterval)
      {
         camera_exposureInterval = (double)(1.0e-3);
      }
      else if(*output == camera_exposureLength)
      {
         camera_exposureLength = (double)(1.0e-6);
      }
      else if(*output == camera_sampleInterval)
      {
         camera_sampleInterval = (double)(0.0);
      }
      else if(*output == splitter_outgoingIncident)
      {
         splitter_outgoingIncident = (WaveTrain)(WaveTrain());
      }
   }
};

#endif // TUTI_SYSTEM_CLASS
