// tempus 2000.11

//
// tempus, version 2000.11, (c) Copyright 1995-2000, MZA Associates Corporation
// Generated on: Wed Jul 18 07:48:22 MDT 2001
//

#include "mliConv.h"
#include "mliSystem.h"
#include "TutA.h"


const int numParams   = 6;
const int numRecords = 2;
const char* sysRecordItems[numRecords] = {
   "targetboard.integrated_intensity  Grid<float>  true  (float)0.0  0.0",
   "uw.power  float  true 0 0"
};

const int numDefRecords = 1;
const char* sysDefRecordItems[numDefRecords] = {
   "targetboard.integrated_intensity  Grid<float>  true  (float)0.0  0.0"
};
mxArray* mliLoadRecords()
{
   mxArray* a = mxCreateCellMatrix(1, numRecords);
   for(int ic=0; ic < numRecords; ic++) {
      mxSetCell(a, ic, mxCreateString(sysRecordItems[ic]));
   }
   return a;
}

mxArray* mliLoadDefRecords()
{
   mxArray* a = mxCreateCellMatrix(1, numDefRecords);
   for(int ic=0; ic < numDefRecords; ic++) {
      mxSetCell(a, ic, mxCreateString(sysDefRecordItems[ic]));
   }
   return a;
}

const char* sysParams[numParams] = {
   "target_range",
   "tel_focus_range",
   "tel_diam",
   "hel_wavelength",
   "tbd_nxy",
   "tbd_dxy"
};

const char* sysParamsVals[numParams] = {
   "float  target_range  50000",
   "float  tel_focus_range  target_range",
   "float  tel_diam  0.75",
   "float  hel_wavelength  1.0e-6",
   "int  tbd_nxy  100",
   "float  tbd_dxy  0.01"
};

/* MEX Functionhandlers */

mxArray* mliLoadParams()
{
   mxArray* a = mxCreateStructMatrix(1, 1, numParams, sysParams);

   mxSetField(a, 0, sysParams[0], mxCreateString(sysParamsVals[0]));
   mxSetField(a, 0, sysParams[1], mxCreateString(sysParamsVals[1]));
   mxSetField(a, 0, sysParams[2], mxCreateString(sysParamsVals[2]));
   mxSetField(a, 0, sysParams[3], mxCreateString(sysParamsVals[3]));
   mxSetField(a, 0, sysParams[4], mxCreateString(sysParamsVals[4]));
   mxSetField(a, 0, sysParams[5], mxCreateString(sysParamsVals[5]));

   return a;
}

class MLITutARunsetA
{
public:

   // Run variables

   // Parameters
   float  target_range;
   float  tel_focus_range;
   float  tel_diam;
   float  hel_wavelength;
   int  tbd_nxy;
   float  tbd_dxy;

   // External Inputs

   // Member functions
   MLITutARunsetA(mxArray* mliparams,
                   float  _target_range,
                   float  _tel_focus_range,
                   float  _tel_diam,
                   float  _hel_wavelength,
                   int  _tbd_nxy,
                   float  _tbd_dxy) :
                   target_range(_target_range),
                   tel_focus_range(_tel_focus_range),
                   tel_diam(_tel_diam),
                   hel_wavelength(_hel_wavelength),
                   tbd_nxy(_tbd_nxy),
                   tbd_dxy(_tbd_dxy)
   {
   }
   ~MLITutARunsetA()
   {
   }
   MLISystemHandle* createSystem(mxArray* filename, mxArray* verbose)
   {
      mxArray* mlia = NULL;

      double stopTime = 0.002;

      bool verb = _mliGetVerbose(verbose);
      char *trffile;
#ifndef NO_TEMPUS_SMF_MONITOR
      char *smfile;
      TempusStatusSMFWriter* smfWriter = mliCreateSMF(filename, &trffile, &smfile);
#else
      mliCreateTRF(filename, &trffile)
#endif
      Universe* mliuniverse = new Universe("TutA");
      TutA* mlisystem = new TutA(NULL, "TutA",
                                 target_range,
                                 tel_focus_range,
                                 tel_diam,
                                 hel_wavelength,
                                 tbd_nxy,
                                 tbd_dxy);

      // Connect external inputs
      MLISystemHandle* handle = new MLISystemHandle(mliuniverse, mlisystem, this);
      handle->SetTrfFileName(trffile);
#ifndef NO_TEMPUS_SMF_MONITOR
      handle->SetSMFFileName(smfile);
      handle->SetSMFWriter(smfWriter);
#endif
      if (handle->recorderFile == NULL) {
         writeParam(handle->pset, "stopTime", DOUBLE, &stopTime);
         writeParam(handle->pset, "target_range", FLOAT, &target_range);
         writeParam(handle->pset, "tel_focus_range", FLOAT, &tel_focus_range);
         writeParam(handle->pset, "tel_diam", FLOAT, &tel_diam);
         writeParam(handle->pset, "hel_wavelength", FLOAT, &hel_wavelength);
         writeParam(handle->pset, "tbd_nxy", INT, &tbd_nxy);
         writeParam(handle->pset, "tbd_dxy", FLOAT, &tbd_dxy);
         handle->OpenRecorderFile();
         if (handle->recorderFile == NULL) {
            if(verb)
               mexPrintf("Error -- Null recorder file may cause problems.\n");
         }
      }
      return(handle);
   }
};

MLITutARunsetA* mliTutARunsetACompParams(mxArray* mliparamsin, mxArray*& mliparamsout, mxArray* mliverbose)
{
   bool verb = _mliGetVerbose(mliverbose);
   mxArray* mlia = NULL;

   double stopTime = 0.002;

   float  target_range = 50000;
   if(mliparamsin) mlia = mxGetField(mliparamsin, 0, "target_range");
   if(!_mliCheckValue(mlia, sysParamsVals[0])) fromMxArray(mlia, target_range);

   float  tel_focus_range = target_range;
   if(mliparamsin) mlia = mxGetField(mliparamsin, 0, "tel_focus_range");
   if(!_mliCheckValue(mlia, sysParamsVals[1])) fromMxArray(mlia, tel_focus_range);

   float  tel_diam = 0.75;
   if(mliparamsin) mlia = mxGetField(mliparamsin, 0, "tel_diam");
   if(!_mliCheckValue(mlia, sysParamsVals[2])) fromMxArray(mlia, tel_diam);

   float  hel_wavelength = 1.0e-6;
   if(mliparamsin) mlia = mxGetField(mliparamsin, 0, "hel_wavelength");
   if(!_mliCheckValue(mlia, sysParamsVals[3])) fromMxArray(mlia, hel_wavelength);

   int  tbd_nxy = 100;
   if(mliparamsin) mlia = mxGetField(mliparamsin, 0, "tbd_nxy");
   if(!_mliCheckValue(mlia, sysParamsVals[4])) fromMxArray(mlia, tbd_nxy);

   float  tbd_dxy = 0.01;
   if(mliparamsin) mlia = mxGetField(mliparamsin, 0, "tbd_dxy");
   if(!_mliCheckValue(mlia, sysParamsVals[5])) fromMxArray(mlia, tbd_dxy);

   mliparamsout = mxCreateStructMatrix(1, 1, numParams, sysParams);

   mxSetField(mliparamsout, 0, "target_range", toMxArray(target_range));
   mxSetField(mliparamsout, 0, "tel_focus_range", toMxArray(tel_focus_range));
   mxSetField(mliparamsout, 0, "tel_diam", toMxArray(tel_diam));
   mxSetField(mliparamsout, 0, "hel_wavelength", toMxArray(hel_wavelength));
   mxSetField(mliparamsout, 0, "tbd_nxy", toMxArray(tbd_nxy));
   mxSetField(mliparamsout, 0, "tbd_dxy", toMxArray(tbd_dxy));

   return(new MLITutARunsetA(mliparamsout,
                             target_range,
                             tel_focus_range,
                             tel_diam,
                             hel_wavelength,
                             tbd_nxy,
                             tbd_dxy));
}
mxArray* mliCompParams(mxArray* paramsin, mxArray* verbose)
{
   mxArray* paramsout;
   MLITutARunsetA* tmpcls = mliTutARunsetACompParams(paramsin, paramsout, verbose);
   delete tmpcls;
   return paramsout;
}

mxArray* mliCreateSystem(mxArray* paramsin, mxArray* filename, mxArray* verbose)
{
   mxArray* paramsout;
   mxArray* runhandle;
   MLITutARunsetA* tmpcls = mliTutARunsetACompParams(paramsin, paramsout, verbose);
   MLISystemHandle* handle = tmpcls->createSystem(filename, verbose);
   runhandle = mliMakePointer(handle);
   return(runhandle);
}

mxArray* mliRecordOutputs(mxArray* mhandle,  mxArray* outputs,
                          mxArray* mstatus,  mxArray* matLeast,  mxArray* mexactly,
                          mxArray* verbose)
{
   if (!_mliVerifyOutputs(outputs)) {
      mexPrintf("Outputs should be a list of structures.\n");
      return mhandle;
   }
   bool verb = _mliGetVerbose(verbose);
   mxClassID outType = mxGetClassID(outputs);
   int nOutputs = 0;
   char outstr[256] = "";
   mxArray* cname = outputs;
   bool   status      = true;
   float  atLeastRate = 0.0;
   double exactlyRate = 0.0;

   MLISystemHandle* handle = reinterpret_cast<MLISystemHandle*>(mliGetPointer(mhandle));
   TutA* system = reinterpret_cast<TutA*>(handle->system);

   if(outType == mxCELL_CLASS)
      nOutputs = mxGetN(outputs);
   else nOutputs = 1;

   handle->NewRecorders(nOutputs);

   for (int ic = 0; ic < nOutputs; ic++) 
   {
      atLeastRate = mxGetPr(matLeast)[ic];
      exactlyRate = mxGetPr(mexactly)[ic];
      status      = mxGetPr(mstatus) [ic] > 0.0;
      if (outType == mxCELL_CLASS) cname = mxGetCell(outputs, ic); 

      if (mxGetString(cname, outstr, 256) != 0) {
         mexPrintf("Error -- Not a string.\n");
      }
      if(strcmp(outstr, "targetboard.integrated_intensity") == 0) {
         if(verb) mexPrintf("Creating recorder %s\n", outstr);

         GridRecorder<float>*  rfsetA1 = new GridRecorder<float>(NULL, "rfsetA1", "targetboard.integrated_intensity", "Grid<float>", "Integrated intensity", status, atLeastRate, exactlyRate);
         if(handle->recorderFile != NULL)
            rfsetA1->dr <<= handle->recorderFile->dr;
         rfsetA1->i <<= system->targetboard.integrated_intensity;
         handle->AddRecorder(rfsetA1);
      }
      if(strcmp(outstr, "uw.power") == 0) {
         if(verb) mexPrintf("Creating recorder %s\n", outstr);

         floatRecorder*  rfsetA2 = new floatRecorder(NULL, "rfsetA2", "uw.power", "float", "Total power (W)", status, atLeastRate, exactlyRate);
         if(handle->recorderFile != NULL)
            rfsetA2->dr <<= handle->recorderFile->dr;
         rfsetA2->i <<= system->uw.power;
         handle->AddRecorder(rfsetA2);
      }
      if (verb) mexPrintf("Output %s not found.\n", outstr);
   }

   return mhandle;
}

mxArray* mliStopRecOutputs(mxArray* mhandle, mxArray* outputs, mxArray* verbose)
{
   if (!_mliVerifyOutputs(outputs)) {
      mexPrintf("Outputs should be a list of structures.\n");
      return mhandle;
   }  
   bool verb = _mliGetVerbose(verbose);
   int       nOutputs = mxGetN(outputs);
   mxClassID outType  = mxGetClassID(outputs);
   char      outstr[256] = "";
   mxArray*  cname = outputs;
   MLISystemHandle* handle = reinterpret_cast<MLISystemHandle*>(mliGetPointer(mhandle));
   for (int ic = 0; ic < nOutputs; ic++) 
   {
      RecordableObject* recobj = NULL;
      if (outType == mxCELL_CLASS) cname = mxGetCell(outputs, ic);
      if (mxGetString(cname, outstr, 256) == 0) 
          recobj = handle->RemoveRecorder(outstr);  
      if (recobj == NULL) {
         if (verb) mexPrintf("Cannot find recorder %s.\n", outstr);
         continue;
      }
      if(strcmp(outstr, "targetboard.integrated_intensity") == 0) {
         if(verb) mexPrintf("Detaching recorder %s\n", outstr);

         GridRecorder<float>*  rfsetA1= (GridRecorder<float>*)reinterpret_cast< RecordableObject* >(recobj);
           rfsetA1->i.detach();
      }
      if(strcmp(outstr, "uw.power") == 0) {
         if(verb) mexPrintf("Detaching recorder %s\n", outstr);

         floatRecorder*  rfsetA2= (floatRecorder*)reinterpret_cast< RecordableObject* >(recobj);
           rfsetA2->i.detach();
      }
   }
   return mhandle;
}

mxArray* mliGetOutput(mxArray* mhandle, mxArray* output, mxArray* verbose)
{
   bool verb = _mliGetVerbose(verbose);
   MLISystemHandle* handle = reinterpret_cast<MLISystemHandle*>(mliGetPointer(mhandle));
   TutA* system = reinterpret_cast<TutA*>(handle->system);
   char outstr[256] = "";
   if (mxGetString(output, outstr, 256) != 0) {
      mexPrintf("Output name should be a string.\n");
   }  
   if(strcmp(outstr, "targetboard.integrated_intensity") == 0) {
      return toMxArray(system->targetboard.integrated_intensity.value());
   }
   if(strcmp(outstr, "uw.power") == 0) {
      return toMxArray(system->uw.power.value());
   }
   mexPrintf("Output %s not found.\n", outstr);
   return toMxArray(0);
}
mxArray* mliSetInputs(mxArray* mhandle, mxArray* inputs, mxArray* values, mxArray* verbose)
{
   bool verb = _mliGetVerbose(verbose);
   mxClassID inType = mxGetClassID(inputs);
   mxClassID valType = mxGetClassID(values);
   int nInputs = 0;
   char instr[256] = "";
   mxArray* cname = inputs;
   mxArray* val = values;
   
   MLISystemHandle* handle = reinterpret_cast<MLISystemHandle*>(mliGetPointer(mhandle));
   MLITutARunsetA* runset = reinterpret_cast<MLITutARunsetA*>(handle->parameters);
   
   if(inType == mxCELL_CLASS) nInputs = mxGetN(inputs);
   else nInputs = 1;

   for(int ic = 0; ic < nInputs; ic++)
   {
      if(inType == mxCELL_CLASS) cname = mxGetCell(inputs, ic);
      if(valType == mxCELL_CLASS) val = mxGetCell(values, ic);
      
      if(mxGetString(cname, instr, 256) != 0) mexPrintf("Error -- Not a string.\n");

   }
   return mhandle;
}
